Package: QuaternaryProd
Type: Package
Title: Computes the Quaternary Dot Product Scoring Statistic for Signed
        and Unsigned Causal Graphs
Version: 1.47.0
Date: 2015-10-22
Authors@R: c(
    person("Carl Tony", "Fakhry", email = "cfakhry@cs.umb.edu", role = c("cre","aut")),
    person("Ping", "Chen", role = "ths"),
    person("Kourosh", "Zarringhalam", role = c("aut","ths")))
Description: QuaternaryProd is an R package that performs causal reasoning on biological 
             networks, including publicly available networks such as STRINGdb. QuaternaryProd 
             is an open-source alternative to commercial products such as Inginuity Pathway 
             Analysis. For a given a set of differentially expressed genes, QuaternaryProd 
             computes the significance of upstream regulators in the network by performing causal
             reasoning using the Quaternary Dot Product Scoring Statistic (Quaternary Statistic),
             Ternary Dot product Scoring Statistic (Ternary Statistic) and Fisher's exact test
	     (Enrichment test). The Quaternary Statistic handles signed, unsigned and ambiguous
             edges in the network. Ambiguity arises when the direction of causality is unknown,
             or when the source node (e.g., a protein) has edges with conflicting signs for the 
	     same target gene. On the other hand, the Ternary Statistic provides causal reasoning
             using the signed and unambiguous edges only. The Vignette provides more details on the
             Quaternary Statistic and illustrates an example of how to perform causal reasoning
             using STRINGdb. 
License: GPL (>=3)
biocViews: GraphAndNetwork, GeneExpression, Transcription
Depends: R (>= 3.2.0), Rcpp (>= 0.11.3), dplyr, yaml (>= 2.1.18)
Suggests: knitr
LinkingTo: Rcpp
VignetteBuilder: knitr
RoxygenNote: 6.1.0
Encoding: UTF-8
git_url: https://git.bioconductor.org/packages/QuaternaryProd
git_branch: devel
git_last_commit: b357c3c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-18
NeedsCompilation: yes
Packaged: 2026-05-18 23:09:11 UTC; biocbuild
Author: Carl Tony Fakhry [cre, aut],
  Ping Chen [ths],
  Kourosh Zarringhalam [aut, ths]
Maintainer: Carl Tony Fakhry <cfakhry@cs.umb.edu>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-19 05:04:32 UTC; unix
