# Generated by roxygen2: do not edit by hand

export(QP_Pmf)
export(QP_Probability)
export(QP_Pvalue)
export(QP_SigPvalue)
export(QP_Support)
export(RunCRE_HSAStringDB)
import(dplyr)
import(yaml)
importFrom(Rcpp,evalCpp)
importFrom(stats,complete.cases)
importFrom(stats,na.omit)
importFrom(utils,read.table)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(QuaternaryProd)
