# QuaternaryProd
An R package for computing the Quaternary Dot Product Scoring Statistic for signed and unsigned causal graphs.

## Installation
Before installing this package, make sure you have the latest version of *Rstudio*, *R* and the *devtools* package. You can install this R pacakge using the following:
```{R}
library(devtools)
install_github("carltonyfakhry/QuaternaryProd", build_vignettes = TRUE, local = FALSE)
```
## Usage
For an introduction to the Quaternary Dot Product Scoring Statistic and for an example on how to compute it over the publicly available network *STRINGdb*, please see 
the *Vignette* for this package using the following:
```{R}
browseVignettes("QuaternaryProd")
```
