#' @importFrom stats na.omit complete.cases
#' @importFrom utils read.table setTxtProgressBar txtProgressBar
#' @import dplyr
#' @import yaml
#' @importFrom Rcpp evalCpp 
#' @useDynLib QuaternaryProd
NULL
