# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

Pvalue_4by3 <- function(q_p, q_m, q_r, q_z, n_p, n_m, n_z, target_score, epsilon) {
    .Call('_QuaternaryProd_Pvalue_4by3', PACKAGE = 'QuaternaryProd', q_p, q_m, q_r, q_z, n_p, n_m, n_z, target_score, epsilon)
}

SigPvalue_4by3 <- function(q_p, q_m, q_r, q_z, n_p, n_m, n_z, target_score, epsilon, significance_level) {
    .Call('_QuaternaryProd_SigPvalue_4by3', PACKAGE = 'QuaternaryProd', q_p, q_m, q_r, q_z, n_p, n_m, n_z, target_score, epsilon, significance_level)
}

domain_4by3 <- function(q_p, q_m, q_r, q_z, n_p, n_m, n_z) {
    .Call('_QuaternaryProd_domain_4by3', PACKAGE = 'QuaternaryProd', q_p, q_m, q_r, q_z, n_p, n_m, n_z)
}

pmf_4by3 <- function(q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon) {
    .Call('_QuaternaryProd_pmf_4by3', PACKAGE = 'QuaternaryProd', q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon)
}

user_probability_of_score_4by3 <- function(target_score, q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon) {
    .Call('_QuaternaryProd_user_probability_of_score_4by3', PACKAGE = 'QuaternaryProd', target_score, q_p, q_m, q_r, q_z, n_p, n_m, n_z, epsilon)
}

